<?php

namespace FM\Factory;

use Doctrine\ORM\EntityManager;
use Teknisa\Model\Traits\TQuery;

class CalendarioFactory
{

    use TQuery;

    protected $connection;

    protected $entityManager;

    public function __construct(EntityManager $entityManager)
    {
        $this->connection = $entityManager->getConnection();
        $this->entityManager = $entityManager;
    }

    public function getTarefasCalendario($parametros, $listaOperadores)
    {
        $sql = "SELECT 
    TO_DATE(FCM_CHECKLISTAGEND.DTHORAAGEND, 'DD/MM/YYYY HH24:MI:SS') AS DTHORAAGEND, 
    'Tarefa' || ' - ' || FCM_CHECKLISTAGEND.NMCHECKLISTAGEND AS NOME, 
    TO_DATE(FCM_CHECKLISTAGEND.DTHRFINALAGEND, 'DD/MM/YYYY HH24:MI:SS') AS DTHRFINALAGEND,
    FCM_LOCALCLI.DSCORIDLOCAL, 
    FCM_CHECKLISTAGEND.NRCHECKLISTAGEND AS CODIGO, 
    FCM_LOCALCLI.NMLOCALCLIENTE, 
    FCM_EQUIPE.NMEQUIPE AS EQUIPE,
    OPERADOR.NMOPERADOR, 
    CLIENTE.NMRAZSOCCLIE || ' - ' || CLIENTE.NMFANTCLIE AS CLIENTE,
    CASE 
        WHEN FCM_CHECKLISTAGEND.IDSITUACAOAGEND = '1' AND FCM_CHECKLISTAGEND.DTHORAAGEND <= SYSDATE THEN 'Atrasado'
        WHEN FCM_CHECKLISTAGEND.IDSITUACAOAGEND = '2' THEN 'Em andamento'
        WHEN FCM_CHECKLISTAGEND.IDSITUACAOAGEND = '3' THEN 'Cancelado'
        WHEN FCM_CHECKLISTAGEND.IDSITUACAOAGEND = '4' THEN 'Concluído'
        WHEN FCM_CHECKLISTAGEND.IDSITUACAOAGEND = '5' THEN 'Atrasado'
        WHEN FCM_CHECKLISTAGEND.IDSITUACAOAGEND = '6' THEN 'Aprovação pendente'
        ELSE NULL 
    END AS SITUACAO
FROM 
    FCM_CHECKLISTAGEND
JOIN 
    FCM_LOCALCLI ON FCM_LOCALCLI.NRLOCALCLIENTE = FCM_CHECKLISTAGEND.NRLOCALCLIENTE
JOIN
    CLIENTE ON FCM_LOCALCLI.CDCLIENTE = CLIENTE.CDCLIENTE
LEFT JOIN 
    OPERADOR ON OPERADOR.CDOPERADOR = FCM_CHECKLISTAGEND.CDOPERADOR
LEFT JOIN 
    FCM_EQUIPETAREFA ON FCM_EQUIPETAREFA.NRCHECKLISTAGEND = FCM_CHECKLISTAGEND.NRCHECKLISTAGEND
LEFT JOIN 
    FCM_EQUIPE ON FCM_EQUIPE.NREQUIPE = FCM_EQUIPETAREFA.NREQUIPE
WHERE 
    (TRUNC(FCM_CHECKLISTAGEND.DTHORAAGEND) BETWEEN TO_DATE(:PERIODOINICIAL, 'DD/MM/YYYY') 
    AND NVL(TO_DATE(:PERIODOFINAL, 'DD/MM/YYYY'), TO_DATE('31/12/2099', 'DD/MM/YYYY'))) -- Filtro do período
    AND (FCM_LOCALCLI.NRLOCALCLIENTEPAI IN (
            SELECT FCM_LOCALCLI.NRLOCALCLIENTE 
            FROM FCM_LOCALCLI 
            WHERE FCM_LOCALCLI.NRLOCALCLIENTEPAI = :NRLOCALCLIENTEPAI
        ) OR :NRLOCALCLIENTEPAI IS NULL) -- Filtro do local superior
    AND (FCM_CHECKLISTAGEND.NRLOCALCLIENTE = :NRLOCALCLIENTE OR :NRLOCALCLIENTE IS NULL) -- Filtro de local
    AND (CLIENTE.CDCLIENTE = :CDCLIENTE OR :CDCLIENTE IS NULL) -- filtro de cliente
    AND (FCM_LOCALCLI.NRTIPOLOCALCLI = :NRTIPOLOCALCLI OR :NRTIPOLOCALCLI IS NULL) -- Filtro de tipo de local
    AND (FCM_EQUIPE.NREQUIPE = :NREQUIPE OR :NREQUIPE IS NULL) -- Filtro de equipe
    AND ((FCM_CHECKLISTAGEND.CDOPERADOR IN ({$listaOperadores})) -- Tarefas criadas para operador específico
        OR (FCM_CHECKLISTAGEND.CDOPERADOR IS NULL AND FCM_CHECKLISTAGEND.IDTAREFAEQUIPE = 'N') -- Tarefas sem operador
        OR (FCM_EQUIPETAREFA.NREQUIPE IN (
                SELECT DISTINCT FCM_EQUIPE.NREQUIPE 
                FROM FCM_EQUIPE
                JOIN FCM_RELACEQUIPEOPER ON FCM_RELACEQUIPEOPER.NREQUIPE = FCM_EQUIPE.NREQUIPE
                WHERE FCM_RELACEQUIPEOPER.CDOPERADOR IN ({$listaOperadores})
            )) -- Tarefas de equipe
        OR FCM_CHECKLISTAGEND.CDOPERINCLUSAO = :CDOPERADOR -- Operador de inclusão
    )

UNION ALL

SELECT 
    TO_DATE(FCM_GERENCIAMENTOCHK.DTHORAAGEND, 'DD/MM/YYYY HH24:MI:SS') AS DTHORAAGEND, 
    'Vistoria' || ' - ' || FCM_GERENCIAMENTOCHK.NMGERENCIAMENTOCHK AS NOME, 
    TO_DATE(FCM_GERENCIAMENTOCHK.DTHRFINALAGEND, 'DD/MM/YYYY HH24:MI:SS') AS DTHRFINALAGEND,
    FCM_LOCALCLI.DSCORIDLOCAL, 
    FCM_GERENCIAMENTOCHK.NRGERENCIAMENTOCHK AS CODIGO, 
    FCM_LOCALCLI.NMLOCALCLIENTE, 
    CLIENTE.NMRAZSOCCLIE || ' - ' || CLIENTE.NMFANTCLIE AS CLIENTE,
    NULL AS EQUIPE,
    OPERADOR.NMOPERADOR,
    CASE 
        WHEN FCM_GERENCIAMENTOCHK.IDSITUACAOGERCHK = '1' AND FCM_CHECKLISTAGEND.IDSITUACAOAGEND = '1' THEN 'Tarefa Pendente'
        WHEN FCM_GERENCIAMENTOCHK.IDSITUACAOGERCHK = '1' AND (FCM_CHECKLISTAGEND.IDSITUACAOAGEND IS NULL OR FCM_CHECKLISTAGEND.IDSITUACAOAGEND = '3') THEN 'Aguardando Vistoria'
        WHEN FCM_GERENCIAMENTOCHK.IDSITUACAOGERCHK = '2' THEN 'Em andamento'
        WHEN FCM_GERENCIAMENTOCHK.IDSITUACAOGERCHK = '3' THEN 'Avaliado'
        WHEN FCM_GERENCIAMENTOCHK.IDSITUACAOGERCHK = '5' THEN 'Cancelado'
        ELSE NULL 
    END AS SITUACAO
FROM 
    FCM_GERENCIAMENTOCHK
LEFT JOIN FCM_CHECKLISTAGEND
    ON FCM_CHECKLISTAGEND.NRCHECKLISTAGEND = FCM_GERENCIAMENTOCHK.NRCHECKLISTAGEND
JOIN 
    FCM_LOCALCLI ON FCM_LOCALCLI.NRLOCALCLIENTE = FCM_GERENCIAMENTOCHK.NRLOCALCLIENTE
JOIN
    CLIENTE ON FCM_LOCALCLI.CDCLIENTE = CLIENTE.CDCLIENTE
JOIN 
    OPERADOR ON OPERADOR.CDOPERADOR = FCM_GERENCIAMENTOCHK.CDOPERADOR
WHERE 
    (TRUNC(FCM_GERENCIAMENTOCHK.DTHORAAGEND) BETWEEN TO_DATE(:PERIODOINICIAL, 'DD/MM/YYYY') 
    AND NVL(TO_DATE(:PERIODOFINAL, 'DD/MM/YYYY'), TO_DATE('31/12/2099', 'DD/MM/YYYY')))
    AND (FCM_LOCALCLI.NRLOCALCLIENTEPAI IN (
            SELECT FCM_LOCALCLI.NRLOCALCLIENTE 
            FROM FCM_LOCALCLI 
            WHERE FCM_LOCALCLI.NRLOCALCLIENTEPAI = :NRLOCALCLIENTEPAI
        ) OR :NRLOCALCLIENTEPAI IS NULL)
    AND (FCM_GERENCIAMENTOCHK.NRLOCALCLIENTE = :NRLOCALCLIENTE OR :NRLOCALCLIENTEPAI IS NULL)
    AND (CLIENTE.CDCLIENTE = :CDCLIENTE OR :CDCLIENTE IS NULL)
    AND (FCM_LOCALCLI.NRTIPOLOCALCLI = :NRTIPOLOCALCLI OR :NRTIPOLOCALCLI IS NULL)
    AND FCM_GERENCIAMENTOCHK.CDOPERADOR IN ({$listaOperadores})
    AND (OPERADOR.CDOPERADOR = :CDOPERADOR OR :CDOPERADOR IS NULL)";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $parametros);
    }

    public function getLocaisCalendario($parametros)
    {

        $sql = "SELECT FCM_LOCALCLI.NRLOCALCLIENTE, FCM_LOCALCLI.NMLOCALCLIENTE,
                             FCM_LOCALCLI.NRSETOR, FCM_LOCALCLI.NRLOCALCLIENTEPAI, FCM_TIPOLOCALCLI.NRTIPOLOCALCLI
                       FROM FCM_LOCALCLI
                       JOIN FCM_TIPOLOCALCLI
                         ON FCM_LOCALCLI.NRTIPOLOCALCLI = FCM_TIPOLOCALCLI.NRTIPOLOCALCLI
                          WHERE FCM_TIPOLOCALCLI.IDTIPOLOCALCLI = :IDTIPOLOCALCLI";


        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $parametros);
    }

    public function getSetoresCalendario()
    {

        $sql = "SELECT SECTOR.NRSETOR, SECTOR.NMSETOR, SECTOR.IDATIVO, SECTOR.NRORG, SECTOR.DTINCLUSAO,
                    SECTOR.CDOPERINCLUSAO, SECTOR.NRORGINCLUSAO, SECTOR.DTULTATU, SECTOR.CDOPERULTATU,
                    SECTOR.NRORGULTATU, SECTOR.NRFUNCRESPONSAVEL 
                    FROM FCM_SETOR SECTOR";


        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function getAtividadesCalendario()
    {

        $sql = "SELECT NRATIVIDADE, NMATIVIDADE
                  FROM FCM_ATIVIDADE";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function getTipoDeLocal()
    {

        $sql = "SELECT NRTIPOLOCALCLI, NMTIPOLOCALCLI
                 FROM FCM_TIPOLOCALCLI";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }
}
