import {
    Metadata, Http, IEventParam,
    MemoryDatasource,
} from '@zeedhi/core';
import {
    LoadingService, DialogService,
    Modal,
    ModalService,
} from '@zeedhi/common';
import { TekGrid } from '@zeedhi/teknisa-components-common';
import { Utilities } from '@/util/Utilities';

export class Fm00020000TipoSolicitacoesController {
    public rowDisabled: boolean = false;

    public modalInatividade!: Modal;

    public modalExclusao!: Modal;

    public async criarModais() {
        this.modalInatividade = await ModalService.createFromJson('fm_00020_000_solicitacoes/modal/fm_00020_000_modal_inatividade_tipo_solicitacao', true);
        this.modalExclusao = await ModalService.createFromJson('fm_00020_000_solicitacoes/modal/fm_00020_000_modal_exclusao_tipo_solicitacao', true);
    }

    public destruirModais() {
        this.modalExclusao.destroy();
        this.modalInatividade.destroy();
    }

    public abrirModalInatividade() {
        this.modalInatividade.show();
    }

    public fecharModalInatividade() {
        this.modalInatividade.hide();
    }

    public abrirModalExclusao() {
        this.modalExclusao.show();
    }

    public fecharModalExclusao() {
        this.modalExclusao.hide();
    }

    public addNewRow({ component }: IEventParam<TekGrid>) {
        const newId = 0;
        component.addNewRow({ NRTIPOSOLICITACAO: newId, IDPRIORIDADEPADR: '1', IDATIVO: 'S' }, 'start');
        setTimeout(() => {
            component.enterEdit('0', 'NMTIPOSOLICITACAO');
        }, 100);
    }

    // Inserir Tipo de Solicitação
    public insertRequestType({ component, event }: IEventParam<TekGrid>) {
        event?.preventDefault();
        if (Utilities.validateGrid(component)) {
            LoadingService.show();
            Http.post('/insertRequestType', {
                PARAMS: component.getEditedRows(),
            }).then((response) => {
                if (response.data.length !== 0) {
                    this.montarModalInatividade(response.data);
                } else {
                    Metadata.getInstance<TekGrid>('tipo-solicitacoes-tek-grid').datasource.get();
                    Utilities.showSuccessMessage();
                }
            }).catch((error: any) => {
                Utilities.showErrorMessage(error);
            }).finally(() => {
                LoadingService.hide();
                component.cancelEditedRows();
            });
        }
    }

    public excluirTipoSolicitacoes({ event }: any) {
        Utilities.showConfirmDeleteMessage(event, 'Fm00020000TipoSolicitacoesController.aposConfirmarExcluir', 'Excluir Registro');
    }

    public aposConfirmarExcluir() {
        const component = Metadata.getInstance<TekGrid>('tipo-solicitacoes-tek-grid');
        LoadingService.show();
        Http.post('/excluirTipoSolicitacao', {
            NRS: component.selectedRows.map((element: any) => element.NRTIPOSOLICITACAO),
        }).then((response) => {
            if (response.data.length !== 0) {
                this.montarModalExclusao(response.data);
            } else {
                component.datasource.get();
                Utilities.showSuccessMessage();
            }
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        }).finally(() => {
            component.selectedRows = [];
            LoadingService.hide();
            DialogService.hide();
        });
    }

    public async cancelEditNewRow({ component }: IEventParam<TekGrid>) {
        component.cancelEditedRows();
        component.setFocus();
    }

    public async montarModalInatividade(solicitacoes: any) {
        await this.abrirModalInatividade();
        const grid = Metadata.getInstance<TekGrid>('grid-solicitacoes-conflito-inatividade');
        grid.datasource.data = [];
        (grid.datasource as MemoryDatasource).allData = [];

        solicitacoes.forEach((element: any) => {
            const dados: any = {
                NRSOLICITACAO: element.NRSOLICITACAO,
                NMSOLICITACAO: element.NMSOLICITACAO,
                DTSOLICITACAO: element.DTSOLICITACAO,
                IDSITUACAO: element.IDSITUACAO,
                NRTIPOSOLICITACAO: element.NRTIPOSOLICITACAO,
                NMTIPOSOLICITACAO: element.NMTIPOSOLICITACAO,
            };
            grid.datasource.post(dados);
        });
        grid.datasource.get();
    }

    public async montarModalExclusao(solicitacoes: any) {
        await this.abrirModalExclusao();
        const grid = Metadata.getInstance<TekGrid>('grid-excluir-tipo-solicitacao');
        grid.datasource.data = [];
        (grid.datasource as MemoryDatasource).allData = [];

        solicitacoes.forEach((element: any) => {
            const dados: any = {
                NRSOLICITACAO: element.NRSOLICITACAO,
                NMSOLICITACAO: element.NMSOLICITACAO,
                DTSOLICITACAO: element.DTSOLICITACAO,
                IDSITUACAO: element.IDSITUACAO,
                NRTIPOSOLICITACAO: element.NRTIPOSOLICITACAO,
                NMTIPOSOLICITACAO: element.NMTIPOSOLICITACAO,
            };
            grid.datasource.post(dados);
        });
        grid.datasource.get();
    }
}
