import { Utilities } from '@/util/Utilities';
import {
    DialogService, LoadingService,
} from '@zeedhi/common';
import { Http, IEventParam, Metadata } from '@zeedhi/core';
import { TekGrid } from '@zeedhi/teknisa-components-common';

export class FmtParametrizacaoImpacto {
    public addNewRow({ component }: IEventParam<TekGrid>) {
        const newId = 0;
        component.addNewRow({ NRIMPACTOMANUT: newId }, 'start');
        setTimeout(() => {
            component.enterEdit('0', 'NMIMPACTOMANUT');
        }, 100);
    }

    public saveImpacto({ component, event }: IEventParam<TekGrid>) {
        event?.preventDefault();
        if (Utilities.validateGrid(component)) {
            LoadingService.show();
            Http.post('saveImpacto', {
                ROWS: component.getEditedRows(),
            }).then(() => {
                Utilities.showSuccessMessage();
                component.datasource.get();
            }).catch((error: any) => {
                Utilities.showErrorMessage(error);
            }).finally(() => {
                component.cancelEditedRows();
                component.editingNewRows = false;
                LoadingService.hide();
            });
        }
    }

    public messageConfirmDelete({ event }: any) {
        Utilities.showConfirmDeleteMessage(event, 'FmtParametrizacaoImpacto.deleteImpactos', 'Excluir Impactos');
    }

    public deleteImpactos() {
        const component = Metadata.getInstance<TekGrid>('tek-grid-impacto');

        LoadingService.show();
        Http.post('/deleteImpactos', {
            NRS: component.selectedRows.map((element: any) => element.NRIMPACTOMANUT),
        }).then(() => {
            Utilities.showSuccessMessage();
            component.datasource.get();
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        }).finally(() => {
            component.selectedRows = [];
            LoadingService.hide();
            DialogService.hide();
        });
    }

    public async cancelEditNewRow({ component }: IEventParam<TekGrid>) {
        component.cancelEditedRows();
        component.setFocus();
    }
}
