import {
    FileInput, Form, LoadingService, Modal, ModalService, Select, TextInput, Textarea, Text,
    Tabs,
    SelectMultiple,
    Button,
    Tooltip,
} from '@zeedhi/common';
import {
    Http, IDictionary, IEventParam, Metadata,
} from '@zeedhi/core';
import {
    IterableCarousel, IterableComponentRender, TekGrid,
} from '@zeedhi/teknisa-components-common';
import { FieldGroup } from '@zeedhi/zd-field-group';
import { Utilities } from '@/util/Utilities';
import { SpeechRecognition } from '@zeedhi/zd-speech-recognition-common';
import FileUpload from './FileUpload';

export class FmtSolicitacaoServicoController {
    private modalRecusarSolicitacao!: Modal;

    private modalCriarOrdemServico!: Modal;

    private modalCriarSolicitacaoServico!: Modal;

    private visibilidade: boolean = true;

    private currentRow: any;

    private tituloModalRecusar: string = '';

    private rowsRecusar: any;

    private carouselSolicitacaoVisivel: boolean = false;

    private isMountedComentarios: boolean = false;

    private actionRow: any;

    public btnRecusar: boolean = false;

    public btnCriarOS: boolean = false;

    public modalHistorico!: Modal;

    public currentObs: any;

    public backupResult: any;

    public btnMicrofone: boolean = true;

    get visibilidadeFrame() {
        return !this.visibilidade;
    }

    public mostrarDetalhesSolicitacao({ row }: any) {
        Metadata.getInstance<Text>('titulo-detalhes-solicitacao').text = row.NMSOLICITACAOSERV;
        const inpCliente = Metadata.getInstance<TextInput>('input-cliente-tab-detalhes');
        const inpLocal = Metadata.getInstance<TextInput>('input-local-tab-detalhes');
        const inpEquipamento = Metadata.getInstance<TextInput>('input-equipamento-tab-detalhes');
        const inpTipoServico = Metadata.getInstance<TextInput>('input-tipo-servico-tab-detalhes');
        const inpImpacto = Metadata.getInstance<TextInput>('input-impacto-tab-detalhes');
        const inpDataSolicitacao = Metadata.getInstance<TextInput>('input-data-solicitacao-tab-detalhes');
        const inpSolicitante = Metadata.getInstance<TextInput>('input-solicitante-tab-detalhes');
        const inpDetalhes = Metadata.getInstance<TextInput>('input-detalhes-tab-detalhes');
        const inpEscopo = Metadata.getInstance<TextInput>('input-escopo-tab-detalhes');
        const inpVistoria = Metadata.getInstance<TextInput>('input-vistoria-tab-detalhes');
        this.visibilidade = false;
        this.currentRow = row;

        inpCliente.value = row.NMFANTCLIE;
        inpLocal.value = row.NMLOCALCLIENTE;
        inpEquipamento.value = row.NMEQUIPAMENTO;
        inpTipoServico.value = row.NMCATEGORIAMANUT;
        inpImpacto.value = row.NMIMPACTOMANUT;
        inpDataSolicitacao.value = row.DTSOLICITACAOSERV;
        inpSolicitante.value = row.NMOPERADOR;
        inpDetalhes.value = row.DSSOLICITACAOSERV;
        inpEscopo.value = row.NMESCOPOMANUT;

        if (row.NRGERENCIAMENTOCHK !== null || row.NRCHECKLISTAGEND !== null) {
            inpVistoria.value = `${row.NMGERENCIAMENTOCHK || row.NMCHECKLISTAGEND} - ${row.DTHORAAGEND}`;
        } else {
            inpVistoria.isVisible = false;
        }

        this.montaDadosComentarios();
    }

    public montaDadosComentarios() {
        const component = Metadata.getInstance<Tabs>('tabs-detalhes-solicitacao');

        if (component.tabs[component.activeTab].name === 'tab-comentarios-solicitacao') {
            setTimeout(() => {
                const cardsComentarios = Metadata.getInstance<IterableComponentRender>('iterable-cards-comentario-solicitacao');
                cardsComentarios.datasource.setFilter({
                    NRSOLICITACAOSERV: this.currentRow.NRSOLICITACAOSERV,
                });
            }, 500);
        }
    }

    public carregarFotosSolicitacao({ component }: any) {
        this.getFotosSolicitacao(this.currentRow.NRSOLICITACAOSERV).then((response: any) => {
            const txtSemRes = Metadata.getInstance<Text>('sem-resultado-imagens-solicitacao');
            if (response.data.data.length > 0) {
                component.iterable.datasource.data = [];
                response.data.data.forEach((dados: IDictionary) => {
                    component.iterable.datasource.data.push({
                        NRIMGSOLICITACAOSERV: dados.NRIMGSOLICITACAOSERV,
                        DSCAMINHO: dados.DSCAMINHO,
                    });
                });
                txtSemRes.isVisible = false;
                component.isVisible = true;
            } else {
                txtSemRes.isVisible = true;
                component.isVisible = false;
            }
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        });
    }

    public getFotosSolicitacao(NRROW: any) {
        return Http.post('/getFotosSolicitacao', {
            NRSOLICITACAOSERV: NRROW,
        });
    }

    public saveComentarioSolicitacao() {
        const form = Metadata.getInstance<Form>('form-comentario-tab-solicitacao');

        if (Utilities.validateForm(form)) {
            LoadingService.show();
            Http.post('/insertComentarioSolicitacao', {
                NRSOLICITACAOSERV: this.currentRow.NRSOLICITACAOSERV,
                DSCOMENTARIO: form.value.textareaComentariosSolicitacao,
            }).then(() => {
                Utilities.showSuccessMessage();
                Metadata.getInstance<Textarea>('textareaComentariosSolicitacao').value = null;
                Metadata.getInstance<IterableComponentRender>('iterable-cards-comentario-solicitacao').datasource.get();
            }).catch((error: any) => {
                Utilities.showErrorMessage(error);
            }).finally(() => {
                LoadingService.hide();
            });
        }
    }

    public voltarTelaSolicitacoes() {
        this.visibilidade = true;
    }

    public async abrirModalRecusarSolicitacao() {
        LoadingService.show();
        this.modalRecusarSolicitacao?.destroy();
        this.modalRecusarSolicitacao = await ModalService.createFromJsonPost('', { jsonPath: 'fmt_solicitacoes_servicos/modal/fmt_recusar_solicitacao' });
        this.modalRecusarSolicitacao.show();
        LoadingService.hide();
    }

    public fecharModalRecusarSolicitacao() {
        this.modalRecusarSolicitacao?.destroy();
        this.modalRecusarSolicitacao.hide();
    }

    public recusarSolicitacao({ row }: any) {
        if (row.IDSITUACAO !== '1') {
            Utilities.showDialog('Operação não permitida!', 'warning', 'Não é possível recusar uma Solicitação que não esteja pendente.');
            return;
        }

        this.tituloModalRecusar = 'Recusar Solicitação';
        this.rowsRecusar = [row];
        this.abrirModalRecusarSolicitacao();
    }

    public async abrirModalHistorico() {
        this.modalHistorico?.destroy();
        this.modalHistorico = await ModalService.createFromJson('fmt_solicitacoes_servicos/modal/fmt_historico_sol_servico', true);
        this.modalHistorico.show();
    }

    public fecharModalHistorico() {
        this.modalHistorico.hide();
        this.modalHistorico.destroy();
    }

    public recusarSolicitacoes() {
        const grid = Metadata.getInstance<TekGrid>('grid-solicitacoes-servicos');
        this.tituloModalRecusar = 'Recusar Solicitações';

        const rowsSelected = grid.selectedRows;

        if (rowsSelected.length === 0) {
            Utilities.showDialog('Aviso', 'warning', 'Selecione pelo menos um registro!');
            return;
        }

        const todasPendentes = rowsSelected.every((row) => row.IDSITUACAO === '1');

        if (!todasPendentes) {
            Utilities.showDialog('Operação não permitida!', 'warning', 'Não é possível recusar uma Solicitação que não esteja pendente.');
            return;
        }

        this.rowsRecusar = rowsSelected;
        this.abrirModalRecusarSolicitacao();
    }

    public saveRecusarSolicitacoes() {
        const form = Metadata.getInstance<Form>('form-recusar-solicitacao');
        const grid = Metadata.getInstance<TekGrid>('grid-solicitacoes-servicos');

        if (Utilities.validateForm(form)) {
            LoadingService.show();
            Http.post('/saveRecusarSolicitacoes', {
                ROWS: this.rowsRecusar,
                DSCOMENTARIO: form.value.textareaMotivoRecusa,
            }).then(() => {
                Utilities.showSuccessMessage();
                grid.datasource.get();
            }).catch((error: any) => {
                Utilities.showErrorMessage(error);
            }).finally(() => {
                this.fecharModalRecusarSolicitacao();
                LoadingService.hide();
            });
        }
    }

    public async criarOrdemServico({ row }: any) {
        if (row.IDSITUACAO !== '1') {
            Utilities.showDialog('Operação não permitida!', 'warning', 'Não é possível criar uma Ordem de Serviço a partir de uma Solicitação que não está pendente.');
        } else {
            this.modalCriarOrdemServico?.destroy();
            this.modalCriarOrdemServico = await ModalService.createFromJsonPost('', { jsonPath: 'fmt_solicitacoes_servicos/modal/fmt_criar_ordem_servico' });
            this.modalCriarOrdemServico.show();
            this.currentRow = row;
        }
    }

    public fecharOrdemServico() {
        this.modalCriarOrdemServico?.destroy();
        this.modalCriarOrdemServico.hide();
    }

    public defineRowAction({ row }: any) {
        this.actionRow = row;
    }

    public expandirDadosOrdemServico({ component }: any) {
        const form = Metadata.getInstance<Form>(`form-${component.name}`);
        form.isVisible = !form.isVisible;
        component.iconName = (component.iconName === 'mdi-chevron-down') ? 'mdi-chevron-right' : 'mdi-chevron-down';
    }

    public preencheDetalhesOrdemServico() {
        const inpNrSolicitacao = Metadata.getInstance<TextInput>('input-nr-solicitacao');
        const inpDataSolicitacao = Metadata.getInstance<TextInput>('input-data-solicitacao');
        const inpSolicitante = Metadata.getInstance<TextInput>('input-solicitante');
        const inpImpacto = Metadata.getInstance<TextInput>('input-impacto');
        const selectEquipamento = Metadata.getInstance<Select>('selectEquipamento');

        inpNrSolicitacao.value = this.currentRow.NRSOLICITACAOSERV;
        inpDataSolicitacao.value = this.currentRow.DTSOLICITACAOSERV;
        inpSolicitante.value = this.currentRow.NMOPERADOR;
        inpImpacto.value = this.currentRow.NMIMPACTOMANUT;
        selectEquipamento.setValue(this.currentRow.NREQUIPAMENTO);
    }

    public visibilidadeTecnicos({ component }: any) {
        const selectTecnicos = Metadata.getInstance<TextInput>('inputTecnicos');
        selectTecnicos.isVisible = component.value === 'N';
    }

    public saveOrdemServico() {
        const form = Metadata.getInstance<Form>('form-informacoes-ordem-servico');
        const grid = Metadata.getInstance<TekGrid>('grid-solicitacoes-servicos');

        const tipoResponsavel = Metadata.getInstance<Select>('tipo-responsavel-criar-ordem');
        const profissionalE = Metadata.getInstance<Select>('profissional-Externo-criar-OS-da-SS');

        let cadastroValido: boolean = false;
        let novoProfissional = {};

        if (tipoResponsavel.value === '3') {
            if (profissionalE.datasource.data.length === 0 || profissionalE.value === null) {
                profissionalE.removeValidation('required');
                cadastroValido = Metadata.getInstance<Form>('form-tecnico-externo').validate();

                if (cadastroValido) {
                    novoProfissional = {
                        NMPROFISSIONALEXTERNO: Metadata.getInstance<TextInput>('nome-tecnico-externo-criar-oss-da-ss').value,
                        NMEMPRESA: Metadata.getInstance<TextInput>('nome-empresa-externo-criar-oss-da-ss').value,
                        INSCRICAO: Metadata.getInstance<TextInput>('cpf-tecnico-externo-criar-oss-da-ss').value,
                    };
                } else {
                    return;
                }
            }
        } else {
            cadastroValido = true;
        }

        if (Utilities.validateForm(form) && cadastroValido) {
            LoadingService.show();
            Http.post('/saveOrdemServico', {
                NRSOLICITACAOSERV: this.currentRow.NRSOLICITACAOSERV,
                NMORDEMOSERV: form.value.inputNome,
                NREQUIPAMENTO: form.value.selectEquipamento,
                NRCATEGORIAMANUT: form.value.selectCategoriaManutencao,
                NRTIPOMANUTENCAO: form.value.selectTipoManutencao,
                NRPRIORIDADEMANUT: form.value.selectPrioridade,
                IDSITUACAO: form.value.selectStatus,
                DTORDEMSERV: form.value.dateData,
                IDFUNCEXTERNO: form.value.selectProfissionalExterno,
                TECNICOS: form.value.selectTecnicos,
                DSORDEMSERV: form.value.inputDescricao,
                NRESCOPOS: form.value.selectEscopo,
                SOLICITACAO: true,
                NRPROFISSIONALEXTERNO: profissionalE.value,
                NREQUIPE: Metadata.getInstance<Select>('EQUIPE-criar-OS-da-SS').value,
                CDOPERADOR: Metadata.getInstance<SelectMultiple>('OPERADOR-criar-OS-da-SS').value,
                NRLOCALCLIENTE: Metadata.getInstance<Select>('NRLOCALCLIENTE-criar-solicitacao').value,
                NOVOPROFISSIONAL: novoProfissional,
                NREPI: Metadata.getInstance<Select>('select-epi-criar-oss-da-ss').value,
                NRMODELOPOP: Metadata.getInstance<Select>('select-modelopop-criar-oss-da-ss').value,
            }).then(() => {
                grid.datasource.get();
                Utilities.showSuccessMessage();
            }).catch((error: any) => {
                Utilities.showErrorMessage(error);
            }).finally(() => {
                this.fecharOrdemServico();
                LoadingService.hide();
            });
        }
    }

    public async abrirModalCriarSolicitacaoServico() {
        this.modalCriarSolicitacaoServico?.destroy();
        this.modalCriarSolicitacaoServico = await ModalService.createFromJsonPost('', { jsonPath: 'fmt_solicitacoes_servicos/modal/fmt_criar_solicitacao_servico' });
        this.modalCriarSolicitacaoServico.show();
    }

    public fecharModalCriarSolicitacaoServico() {
        this.modalCriarSolicitacaoServico?.destroy();
        this.modalCriarSolicitacaoServico.hide();
    }

    public async saveSolicitacaoServico() {
        const form = Metadata.getInstance<Form>('form-criar-solicitacao-servico');
        const imgs = await this.updloadPictures('file-input-fotos-solicitacao');
        const grid = Metadata.getInstance<TekGrid>('grid-solicitacoes-servicos');
        const tipoLocal = Metadata.getInstance<Select>('NRTIPOLOCALCLI-criar-solicitacao');
        const local = Metadata.getInstance<Select>('NRLOCALCLIENTE-criar-solicitacao');

        if (Utilities.validateForm(form)) {
            LoadingService.show();
            Http.post('/saveSolicitacaoServico', {
                NMSOLICITACAOSERV: form.value.inputNomeSolicitacao,
                NRCATEGORIAMANUT: form.value.inputTipoServico,
                NREQUIPAMENTO: form.value.inputEquipamento,
                NRIMPACTOMANUT: form.value.inputImpacto,
                DSSOLICITACAOSERV: form.value.inputDescricao,
                CDOPERSOLICITANTE: form.value.selectResponsavel,
                FOTOS: imgs,
                NRTIPOLOCALCLI: tipoLocal.value,
                NRLOCALCLIENTE: local.value,
            }).then(() => {
                grid.datasource.get();
                this.fecharModalCriarSolicitacaoServico();
                Utilities.showSuccessMessage();
            }).catch((error: any) => {
                Utilities.showErrorMessage(error);
            }).finally(() => {
                LoadingService.hide();
            });
        }
    }

    public async preencheCamposSolicitacao() {
        const inpResponsavel = Metadata.getInstance<Select>('selectResponsavel');
        const inpData = Metadata.getInstance<TextInput>('input-data-solicitacao');

        inpResponsavel.setValue(await this.getOperadorLogado());
        inpData.value = (new Date()).toLocaleDateString();
    }

    public mostrarFotoSolicitacao({ component }: any) {
        const carouselFotos = Metadata.getInstance<IterableCarousel>('carouselFotosSolicitacao');
        carouselFotos.iterable.datasource.updateData([]);
        const imagens: any = [];
        component.value.forEach((element: any, pos: any) => {
            imagens.push({
                id: `numero_${pos}`,
                src: window.URL.createObjectURL(element),
            });
        });
        carouselFotos.iterable.datasource.updateData(imagens);
        this.carouselSolicitacaoVisivel = component.value.length > 0;
    }

    public async updloadPictures(name: string) {
        const fileInputObject = Metadata.getInstance<FileInput>(name).value;
        let arquivos: any = [];
        if (fileInputObject && fileInputObject.length > 0) {
            const types: IDictionary = ['image/jpg', 'image/jpeg', 'image/png'];
            const buildFilesPromises = fileInputObject.map((file: any) => {
                if (types.includes(file.type)) {
                    return FileUpload.buildFileSendObject(file);
                }
                return [];
            });
            await Promise.all(buildFilesPromises).then((files: any) => {
                arquivos = files;
            });
        }
        return arquivos;
    }

    public async getOperadorLogado() {
        let cdoper: string = '';
        try {
            const response = await Http.get('/getOperadorLogado');
            cdoper = response.data.CDOPERADOR;
        } catch (error) {
            Utilities.showErrorMessage(error);
        }
        return cdoper;
    }

    public async abrirHistorico() {
        await this.abrirModalHistorico();
        Metadata.getInstance<TekGrid>('grid-historico-solserv').datasource.setFilter({
            NRSOLICITACAOSERV: this.actionRow.NRSOLICITACAOSERV,
        });
    }

    public changeTipoDeLocal({ component }: IEventParam<Select>) {
        const tipoLocal = Metadata.getInstance<Select>('NRTIPOLOCALCLI-criar-solicitacao');
        const local = Metadata.getInstance<Select>('NRLOCALCLIENTE-criar-solicitacao');

        tipoLocal.value = null;
        local.value = null;

        if (component.value) {
            tipoLocal.disabled = false;
            tipoLocal.datasource.setFilter({
                CDCLIENTE: component.value,
            });
        } else {
            tipoLocal.disabled = true;
            local.disabled = true;
        }
    }

    public changeLocal({ component }: IEventParam<Select>) {
        const local = Metadata.getInstance<Select>('NRLOCALCLIENTE-criar-solicitacao');

        if (component.value) {
            local.disabled = false;

            local.datasource.setFilter({
                NRTIPOLOCALCLI: component.value,
                UNICA: 'S',
                CDCLIENTE: Metadata.getInstance<Select>('CDCLIENTE-criar-solicitacao').value,
            });
        } else {
            local.value = null;
            local.disabled = true;
        }
    }

    public changeResponsavel({ component }: IEventParam<Select>) {
        const elements = {
            operador: Metadata.getInstance<SelectMultiple>('OPERADOR-criar-OS-da-SS'),
            equipe: Metadata.getInstance<Select>('EQUIPE-criar-OS-da-SS'),
            profissionalE: Metadata.getInstance<Select>('profissional-Externo-criar-OS-da-SS'),
            fielGroupTecnico: Metadata.getInstance<FieldGroup>('field-group-cadastro-tecnico-externo'),
            local: Metadata.getInstance<Select>('NRLOCALCLIENTE-criar-solicitacao'),

            formTecnicoExterno: Metadata.getInstance<Form>('form-tecnico-externo'),
            btnNovo: Metadata.getInstance<Button>('btn-novo-profissional-externo-criar-oss-da-ss'),
            nome: Metadata.getInstance<TextInput>('nome-tecnico-externo-criar-oss-da-ss'),
            empresa: Metadata.getInstance<TextInput>('nome-empresa-externo-criar-oss-da-ss'),
            cnpj: Metadata.getInstance<TextInput>('cpf-tecnico-externo-criar-oss-da-ss'),
        };

        // Resetar todos os elementos para estado inicial
        elements.btnNovo.label = 'Novo';
        const resetAll = () => {
            this.setElementState(elements.operador, false);
            this.setElementState(elements.equipe, false);
            this.setElementState(elements.profissionalE, false);
            elements.btnNovo.isVisible = false;
            elements.fielGroupTecnico.isVisible = false;
            elements.formTecnicoExterno.clearForm();
            this.setElementState(elements.nome, true, null, null);
            this.setElementState(elements.empresa, true, null, null);
            this.setElementState(elements.cnpj, true, null, null);
        };

        // Aplicar lógica com base no valor do componente
        resetAll();
        switch (component.value) {
            case '1':
                this.setElementState(elements.operador, true, undefined, { type: 'required', params: {} });
                elements.operador.datasource.setFilter({ NRLOCALCLIENTE: elements.local.value });
                break;
            case '2':
                this.setElementState(elements.equipe, true, undefined, { type: 'required', params: {} });
                elements.equipe.datasource.setFilter({ NRLOCALCLIENTE: elements.local.value });
                break;
            case '3':
                this.setElementState(elements.profissionalE, true, undefined, { type: 'required', params: {} });
                this.setElementState(elements.btnNovo, true);
                break;
            default:
                // Default logic handled by resetAll
                break;
        }
    }

    public visibilidadeFieldGroupProfissionalExterno({ component }: IEventParam<Button>) {
        const elements = {
            formTecnicoExterno: Metadata.getInstance<Form>('form-tecnico-externo'),
            fielGroupTecnico: Metadata.getInstance<FieldGroup>('field-group-cadastro-tecnico-externo'),
            nome: Metadata.getInstance<TextInput>('nome-tecnico-externo-criar-oss-da-ss'),
            empresa: Metadata.getInstance<TextInput>('nome-empresa-externo-criar-oss-da-ss'),
            cnpj: Metadata.getInstance<TextInput>('cpf-tecnico-externo-criar-oss-da-ss'),
        };

        const resetAll = () => {
            this.setElementState(elements.nome, true);
            this.setElementState(elements.empresa, true);
            this.setElementState(elements.cnpj, true);
            component.label = 'Novo';
            elements.fielGroupTecnico.isVisible = false;
            elements.formTecnicoExterno.clearForm();
        };

        if (component.label === 'Novo') {
            elements.fielGroupTecnico.isVisible = true;
            this.setElementState(elements.nome, true, undefined, { type: 'required', params: {} });
            this.setElementState(elements.empresa, true, undefined, { type: 'required', params: {} });
            this.setElementState(elements.cnpj, true, undefined, { type: 'required', params: {} });
            component.label = 'Cancelar';
        } else {
            resetAll();
        }
    }

    // Função auxiliar para alterar visibilidade e valor
    public setElementState(element: any, isVisible: boolean, value: any = null, validation: { type: string; params: any } | null = null) {
        element.isVisible = isVisible;
        if (value !== undefined) {
            element.value = value;
        }
        if (validation) {
            element.addValidation(validation.type, validation.params);
        } else {
            element.removeValidation('required');
        }
    }

    public switchIcon({ component }: IEventParam<Button>) {
        const { name } = component;
        this.currentObs = name;

        const instance = Metadata.getInstance<SpeechRecognition>('speech-solicitacao-servico');
        const textIconInstance = Metadata.getInstance<Tooltip>('tooltip-speech-solicitacao-servico');
        instance.toggleRecording();
        component.color = instance.isRecording ? 'red' : 'var(--v-primary-base)';
        textIconInstance.label = instance.isRecording ? 'Desativar gravação por voz' : 'Ativar gravação por voz';
    }

    public onResult({ result }: IEventParam<SpeechRecognition>) {
        const obs = Metadata.getInstance<Textarea>('inputDescricao');

        obs.value = this.backupResult;
        obs.value += ` ${result}`;
    }

    public gerarBackupResult() {
        const obs = Metadata.getInstance<Textarea>('inputDescricao');
        const speech = Metadata.getInstance<SpeechRecognition>('speech-solicitacao-servico');

        this.backupResult = '';
        if (obs.value) {
            this.backupResult = obs.value;
        } else {
            this.backupResult = '';
            speech.reset();
        }
    }

    // SPEECH COMENTAR SOLICITACAO DE SERVICO

    public switchIconSolicitacaoServico({ component }: IEventParam<Button>) {
        const { name } = component;
        this.currentObs = name;

        const instance = Metadata.getInstance<SpeechRecognition>('speech-comentario-solicitacao');
        const textIconInstance = Metadata.getInstance<Tooltip>('tooltip-speech-comentario-solicitacao');
        instance.toggleRecording();
        component.color = instance.isRecording ? 'red' : 'var(--v-primary-base)';
        textIconInstance.label = instance.isRecording ? 'Desativar gravação por voz' : 'Ativar gravação por voz';
    }

    public onResultSolicitacaoServico({ result }: IEventParam<SpeechRecognition>) {
        const obs = Metadata.getInstance<Textarea>('textareaComentariosSolicitacao');

        obs.value = this.backupResult;
        obs.value += ` ${result}`;
    }

    public gerarBackupSolicitacaoServico() {
        const obs = Metadata.getInstance<Textarea>('textareaComentariosSolicitacao');
        const speech = Metadata.getInstance<SpeechRecognition>('speech-comentario-solicitacao');

        this.backupResult = '';
        if (obs.value) {
            this.backupResult = obs.value;
        } else {
            this.backupResult = '';
            speech.reset();
        }
    }
}
